package com.nisum.job;

import com.nisum.impl.ForEachWriterImpl;
import com.nisum.config.SparkSessionConfig;
import com.nisum.constants.IKafkaConstants;
import com.nisum.model.Person;
import io.confluent.kafka.schemaregistry.client.CachedSchemaRegistryClient;
import io.confluent.kafka.schemaregistry.client.SchemaMetadata;
import io.confluent.kafka.schemaregistry.client.rest.exceptions.RestClientException;
import io.confluent.kafka.serializers.KafkaAvroDeserializer;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericRecord;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.sql.SQLContext;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.avro.SchemaConverters;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.types.StructType;
import org.slf4j.LoggerFactory;

import java.io.IOException;

import static org.apache.spark.sql.Encoders.bean;

public class KafkaDataWriterJob {
    private static final SparkSession sparkSession = SparkSessionConfig.sparkSession();

    private static final SQLContext sqlContext = SparkSessionConfig.sparkSession().sqlContext();

    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(KafkaDataWriterJob.class);

    public void run() throws IOException, RestClientException {
        LOGGER.info("Kafka Data Write job got started");

        Logger.getLogger("org.apache").setLevel(Level.WARN);
        Logger.getLogger("org.apache.spark.storage").setLevel(Level.ERROR);
        CachedSchemaRegistryClient cachedSchemaRegistryClient = new CachedSchemaRegistryClient(IKafkaConstants.SCHEMA_REGISTRY_URL, 128);
        SchemaMetadata metadata = cachedSchemaRegistryClient.getLatestSchemaMetadata(IKafkaConstants.TOPIC_NAME + "-value");
        Schema schema = cachedSchemaRegistryClient.getById(metadata.getId());
        StructType dataType = (StructType) SchemaConverters.toSqlType(schema).dataType();

        sparkSession.udf().register("deserialize", (byte[] data) -> {
            CachedSchemaRegistryClient schemaRegistryClient = new CachedSchemaRegistryClient(IKafkaConstants.SCHEMA_REGISTRY_URL, 128);
            KafkaAvroDeserializer kafkaAvroDeserializer = new KafkaAvroDeserializer(schemaRegistryClient);
            GenericRecord record = (GenericRecord) kafkaAvroDeserializer.deserialize("", data);

            return RowFactory.create(record.get("name").toString(), record.get("age").toString()); }, dataType);

        LOGGER.info("Kafka Data Write job started reading data from the specified Topic::{}",IKafkaConstants.TOPIC_NAME);
        Dataset<Person> rowDataSet = sparkSession
                .readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers",IKafkaConstants.KAFKA_SERVER)
                .option("subscribe",IKafkaConstants.TOPIC_NAME)
                .option("startingOffsets", IKafkaConstants.START_OFFSET)
                .option("failOnDataLoss","false")
                .load()
                .selectExpr("deserialize(value) as value")
                .select("value.*")
                .as(bean(Person.class));

        rowDataSet.printSchema();
        rowDataSet.createOrReplaceTempView("personData");

        try {
            LOGGER.info("Kafka Data Write job started writing data to the kafka Topic");

            Dataset<Person> personDataSet = sparkSession.sqlContext().sql("select * from personData").as(bean(Person.class));
            personDataSet.show();
            personDataSet.writeStream()
                    .foreach(new ForEachWriterImpl())
                    .outputMode(OutputMode.Append())
                    .option("checkpointLocation", "/tmp/OfferLookupKafkaCheckpointagain")
                    .start()
                    .awaitTermination();


        } catch ( Exception e ) {
            LOGGER.error("Exception in KafkaData Writer Job :: {}", e.getMessage());
        }
        finally {
            LOGGER.info("Spark Session is closed ");
            sparkSession.close();
        }

    }
}
